==> linux-2.1.130.tar.gz: linux-2.1.130/drivers/block/partbl_osf.c <==
/*
 *  partbl_osf.c -- partition table parser for OSF partitions (alpha)
 *
 *  Code extracted from
 *  linux/drivers/block/genhd.c
 *
 *  Copyright (C) 1991, 1992  Linus Torvalds
 */

#include <linux/module.h>

#include <linux/fs.h>
#include <linux/genhd.h>
#include <linux/kernel.h>
#include <linux/major.h>
#include <linux/string.h>
#include <linux/init.h>


static int osf_partition(struct gendisk *hd, kdev_t dev, unsigned long first_sector)
{
	int i;
	int mask = (1 << hd->minor_shift) - 1;
	struct buffer_head *bh;
	struct disklabel {
		u32 d_magic;
		u16 d_type,d_subtype;
		u8 d_typename[16];
		u8 d_packname[16];
		u32 d_secsize;
		u32 d_nsectors;
		u32 d_ntracks;
		u32 d_ncylinders;
		u32 d_secpercyl;
		u32 d_secprtunit;
		u16 d_sparespertrack;
		u16 d_sparespercyl;
		u32 d_acylinders;
		u16 d_rpm, d_interleave, d_trackskew, d_cylskew;
		u32 d_headswitch, d_trkseek, d_flags;
		u32 d_drivedata[5];
		u32 d_spare[5];
		u32 d_magic2;
		u16 d_checksum;
		u16 d_npartitions;
		u32 d_bbsize, d_sbsize;
		struct d_partition {
			u32 p_size;
			u32 p_offset;
			u32 p_fsize;
			u8  p_fstype;
			u8  p_frag;
			u16 p_cpg;
		} d_partitions[8];
	} * label;
	struct d_partition * partition;
#define DISKLABELMAGIC (0x82564557UL)

	MOD_INC_USE_COUNT;
	if (!(bh = bread(dev,0,get_ptable_blocksize(dev)))) {
		printk("unable to read partition table\n");
	    MOD_DEC_USE_COUNT;
		return -1;
	}
	label = (struct disklabel *) (bh->b_data+64);
	partition = label->d_partitions;
	if (label->d_magic != cpu_to_le32(DISKLABELMAGIC)) {
#if 0
		printk("magic: %08x\n", label->d_magic);
#endif
		brelse(bh);
	    MOD_DEC_USE_COUNT;
		return 0;
	}
	if (label->d_magic2 != cpu_to_le32(DISKLABELMAGIC)) {
#if 0
		printk("magic2: %08x\n", label->d_magic2);
#endif
		brelse(bh);
	    MOD_DEC_USE_COUNT;
		return 0;
	}
	printk(" OSF");
	for (i = 0 ; i < le16_to_cpu(label->d_npartitions); i++, partition++) {
		if ((current_minor & mask) == 0)
		        break;
		if (partition->p_size)
			add_partition(hd, current_minor,
				first_sector+le32_to_cpu(partition->p_offset),
				le32_to_cpu(partition->p_size));
		current_minor++;
	}
	printk("\n");
	brelse(bh);
	MOD_DEC_USE_COUNT;
	return 1;
}

static struct partition_parser osf_partition_parser =
    { NULL, "osf",  osf_partition };

#ifndef MODULE
__initfunc(void osf_partbl_init( void ))
{
	(void)register_partbl( &osf_partition_parser );
}
#else
int init_module(void)
{
	return register_partbl( &osf_partition_parser );
}

void cleanup_module(void)
{
	(void)unregister_partbl( &osf_partition_parser );
}
#endif

==> linux-2.3.6.tar.gz: linux-2.3.6/drivers/block/partbl_osf.c <==
/*
 *  partbl_osf.c -- partition table parser for OSF partitions (alpha)
 *
 *  Code extracted from
 *  linux/drivers/block/genhd.c
 *
 *  Copyright (C) 1991, 1992  Linus Torvalds
 */

#include <linux/module.h>

#include <linux/fs.h>
#include <linux/genhd.h>
#include <linux/kernel.h>
#include <linux/major.h>
#include <linux/string.h>
#include <linux/init.h>


static int osf_partition(struct gendisk *hd, kdev_t dev, unsigned long first_sector)
{
	int i;
	int mask = (1 << hd->minor_shift) - 1;
	struct buffer_head *bh;
	struct disklabel {
		u32 d_magic;
		u16 d_type,d_subtype;
		u8 d_typename[16];
		u8 d_packname[16];
		u32 d_secsize;
		u32 d_nsectors;
		u32 d_ntracks;
		u32 d_ncylinders;
		u32 d_secpercyl;
		u32 d_secprtunit;
		u16 d_sparespertrack;
		u16 d_sparespercyl;
		u32 d_acylinders;
		u16 d_rpm, d_interleave, d_trackskew, d_cylskew;
		u32 d_headswitch, d_trkseek, d_flags;
		u32 d_drivedata[5];
		u32 d_spare[5];
		u32 d_magic2;
		u16 d_checksum;
		u16 d_npartitions;
		u32 d_bbsize, d_sbsize;
		struct d_partition {
			u32 p_size;
			u32 p_offset;
			u32 p_fsize;
			u8  p_fstype;
			u8  p_frag;
			u16 p_cpg;
		} d_partitions[8];
	} * label;
	struct d_partition * partition;
#define DISKLABELMAGIC (0x82564557UL)

	MOD_INC_USE_COUNT;
	if (!(bh = bread(dev,0,get_ptable_blocksize(dev)))) {
		printk("unable to read partition table\n");
	    MOD_DEC_USE_COUNT;
		return -1;
	}
	label = (struct disklabel *) (bh->b_data+64);
	partition = label->d_partitions;
	if (label->d_magic != cpu_to_le32(DISKLABELMAGIC)) {
#if 0
		printk("magic: %08x\n", label->d_magic);
#endif
		brelse(bh);
	    MOD_DEC_USE_COUNT;
		return 0;
	}
	if (label->d_magic2 != cpu_to_le32(DISKLABELMAGIC)) {
#if 0
		printk("magic2: %08x\n", label->d_magic2);
#endif
		brelse(bh);
	    MOD_DEC_USE_COUNT;
		return 0;
	}
	printk(" OSF");
	for (i = 0 ; i < le16_to_cpu(label->d_npartitions); i++, partition++) {
		if ((current_minor & mask) == 0)
		        break;
		if (partition->p_size)
			partbl_add_partition(hd, current_minor,
				first_sector+le32_to_cpu(partition->p_offset),
				le32_to_cpu(partition->p_size));
		current_minor++;
	}
	printk("\n");
	brelse(bh);
	MOD_DEC_USE_COUNT;
	return 1;
}

static struct partition_parser osf_partition_parser =
    { NULL, "osf",  osf_partition };

#ifndef MODULE
__initfunc(void osf_partbl_init( void ))
{
	(void)register_partbl( &osf_partition_parser );
}
#else
int init_module(void)
{
	return register_partbl( &osf_partition_parser );
}

void cleanup_module(void)
{
	(void)unregister_partbl( &osf_partition_parser );
}
#endif
